@@ -257,11 +257,13 @@ class AudioSegment(object):
         """
         return self._data
 
-    def get_array_of_samples(self):
+    def get_array_of_samples(self, array_type_override=None):
         """
         returns the raw_data as an array of samples
         """
-        return array.array(self.array_type, self._data)
+        if array_type_override is None:
+            array_type_override = self.array_type
+        return array.array(array_type_override, self._data)
 
     @property
     def array_type(self):
@@ -279,6 +281,9 @@ class AudioSegment(object):
         except:
             return False
 
+    def __hash__(self):
+       return hash(AudioSegment) ^ hash((self.channels, self.frame_rate, self.sample_width, self._data))
+
     def __ne__(self, other):
         return not (self == other)
 
@@ -669,7 +674,11 @@ class AudioSegment(object):
                 bits_per_sample = 16
             else:
                 bits_per_sample = audio_streams[0]['bits_per_sample']
-            acodec = 'pcm_s%dle' % bits_per_sample
+            if bits_per_sample == 8:
+                acodec = 'pcm_s8'
+            else:
+                acodec = 'pcm_s%dle' % bits_per_sample
+
             conversion_command += ["-acodec", acodec]
 
         conversion_command += [
@@ -704,19 +713,19 @@ class AudioSegment(object):
 
     @classmethod
     def from_mp3(cls, file, parameters=None):
-        return cls.from_file(file, 'mp3', parameters)
+        return cls.from_file(file, 'mp3', parameters=parameters)
 
     @classmethod
     def from_flv(cls, file, parameters=None):
-        return cls.from_file(file, 'flv', parameters)
+        return cls.from_file(file, 'flv', parameters=parameters)
 
     @classmethod
     def from_ogg(cls, file, parameters=None):
-        return cls.from_file(file, 'ogg', parameters)
+        return cls.from_file(file, 'ogg', parameters=parameters)
 
     @classmethod
     def from_wav(cls, file, parameters=None):
-        return cls.from_file(file, 'wav', parameters)
+        return cls.from_file(file, 'wav', parameters=parameters)
 
     @classmethod
     def from_raw(cls, file, **kwargs):
@@ -845,7 +854,7 @@ class AudioSegment(object):
                         "-id3v2_version", id3v2_version
                     ])
 
-        if sys.platform == 'darwin':
+        if sys.platform == 'darwin' and codec == 'mp3':
             conversion_command.extend(["-write_xing", "0"])
 
         conversion_command.extend([
