@@ -2,10 +2,16 @@ from functools import partial
 import os
 import sys
 import unittest
-from tempfile import NamedTemporaryFile
+from tempfile import (
+    NamedTemporaryFile,
+    mkdtemp,
+    gettempdir
+)
+import tempfile
 import struct
 
 from pydub import AudioSegment
+from pydub.audio_segment import extract_wav_headers
 from pydub.utils import (
     db_to_float,
     ratio_to_db,
@@ -50,6 +56,110 @@ class UtilityTests(unittest.TestCase):
         self.assertEqual(12, ratio_to_db(db_to_float(12, using_amplitude=False), using_amplitude=False))
 
 
+if sys.version_info >= (3, 6):
+    class PathLikeObjectTests(unittest.TestCase):
+
+        class MyPathLike:
+            def __init__(self, path):
+                self.path = path
+
+            def __fspath__(self):
+                return self.path
+
+        def setUp(self):
+            self.mp3_path_str = os.path.join(data_dir, 'test1.mp3')
+
+            from pathlib import Path
+            self.mp3_pathlib_path = Path(self.mp3_path_str)
+
+            self.mp3_path_like_str = self.MyPathLike(self.mp3_path_str)
+            self.mp3_path_like_bytes = self.MyPathLike(bytes(self.mp3_path_str, sys.getdefaultencoding()))
+
+        def test_audio_segment_from_pathlib_path(self):
+            seg1 = AudioSegment.from_file(self.mp3_path_str)
+            seg2 = AudioSegment.from_file(self.mp3_pathlib_path)
+
+            self.assertEqual(len(seg1), len(seg2))
+            self.assertEqual(seg1._data, seg2._data)
+            self.assertTrue(len(seg1) > 0)
+
+        def test_audio_segment_from_path_like_str(self):
+            seg1 = AudioSegment.from_file(self.mp3_path_str)
+            seg2 = AudioSegment.from_file(self.mp3_path_like_str)
+
+            self.assertEqual(len(seg1), len(seg2))
+            self.assertEqual(seg1._data, seg2._data)
+            self.assertTrue(len(seg1) > 0)
+
+        def test_audio_segment_from_path_like_bytes(self):
+            seg1 = AudioSegment.from_file(self.mp3_path_str)
+            seg2 = AudioSegment.from_file(self.mp3_path_like_bytes)
+
+            self.assertEqual(len(seg1), len(seg2))
+            self.assertEqual(seg1._data, seg2._data)
+            self.assertTrue(len(seg1) > 0)
+
+        def test_non_existant_pathlib_path(self):
+            from pathlib import Path
+            path = Path('this/path/should/not/exist/do/not/make/this/exist')
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+            path = Path('')
+            # On Unicies this will raise a IsADirectoryError, on Windows this
+            # will result in a PermissionError. Both of these are subclasses of
+            # OSError. We aren't so much worried about the specific exception
+            # here, just that reading a file from an empty path is an error.
+            with self.assertRaises(OSError):
+                _ = AudioSegment.from_file(path)
+
+        def test_non_existant_path_like_str(self):
+            path = self.MyPathLike('this/path/should/not/exist/do/not/make/this/exist')
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+            path = self.MyPathLike('')
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+        def test_non_existant_path_like_bytes(self):
+            path = self.MyPathLike(bytes('this/path/should/not/exist/do/not/make/this/exist', sys.getdefaultencoding()))
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+            path = self.MyPathLike(bytes('', sys.getdefaultencoding()))
+            with self.assertRaises(FileNotFoundError):
+                _ = AudioSegment.from_file(path)
+
+        def assertWithinRange(self, val, lower_bound, upper_bound):
+            self.assertTrue(lower_bound < val < upper_bound,
+                            "%s is not in the acceptable range: %s - %s" %
+                            (val, lower_bound, upper_bound))
+
+        def assertWithinTolerance(self, val, expected, tolerance=None,
+                                  percentage=None):
+            if percentage is not None:
+                tolerance = val * percentage
+            lower_bound = val - tolerance
+            upper_bound = val + tolerance
+            self.assertWithinRange(val, lower_bound, upper_bound)
+
+        def test_export_pathlib_path(self):
+            seg1 = AudioSegment.from_file(self.mp3_path_str)
+            from pathlib import Path
+            path = Path(tempfile.gettempdir()) / 'pydub-test-export-8ajds.mp3'
+            try:
+                seg1.export(path, format='mp3')
+                seg2 = AudioSegment.from_file(path, format='mp3')
+
+                self.assertTrue(len(seg1) > 0)
+                self.assertWithinTolerance(len(seg1),
+                                           len(seg2),
+                                           percentage=0.01)
+            finally:
+                os.unlink(path)
+
+
 class FileAccessTests(unittest.TestCase):
 
     def setUp(self):
@@ -74,6 +184,7 @@ class AudioSegmentTests(unittest.TestCase):
     def setUp(self):
         global test1, test2, test3, testparty, testdcoffset
         if not test1:
+            a = os.path.join(data_dir, 'test1.mp3')
             test1 = AudioSegment.from_mp3(os.path.join(data_dir, 'test1.mp3'))
             test2 = AudioSegment.from_mp3(os.path.join(data_dir, 'test2.mp3'))
             test3 = AudioSegment.from_mp3(os.path.join(data_dir, 'test3.mp3'))
@@ -110,7 +221,8 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertWithinRange(val, lower_bound, upper_bound)
 
     def test_direct_instantiation_with_bytes(self):
-        seg = AudioSegment(b'RIFF\x28\x00\x00\x00WAVEfmt \x10\x00\x00\x00\x01\x00\x02\x00\x00}\x00\x00\x00\xf4\x01\x00\x04\x00\x10\x00data\x04\x00\x00\x00\x00\x00\x00\x00')
+        seg = AudioSegment(
+            b'RIFF\x28\x00\x00\x00WAVEfmt \x10\x00\x00\x00\x01\x00\x02\x00\x00}\x00\x00\x00\xf4\x01\x00\x04\x00\x10\x00data\x04\x00\x00\x00\x00\x00\x00\x00')
         self.assertEqual(seg.frame_count(), 1)
         self.assertEqual(seg.channels, 2)
         self.assertEqual(seg.sample_width, 2)
@@ -129,6 +241,31 @@ class AudioSegmentTests(unittest.TestCase):
         # the data length should have grown by exactly 4:3 (24 bits turn into 32 bits)
         self.assertEqual(len(seg24.raw_data) * 3, len24 * 4)
 
+    def test_192khz_audio(self):
+        test_files = [('test-192khz-16bit.wav', 16),
+                      ('test-192khz-24bit.wav', 32),
+                      ('test-192khz-32bit.flac', 32),
+                      ('test-192khz-32bit.wav', 32),
+                      ('test-192khz-64bit.wav', 64)]
+        base_file, bit_depth = test_files[0]
+        path = os.path.join(data_dir, base_file)
+        base = AudioSegment.from_file(path)
+
+        headers = extract_wav_headers(open(path, 'rb').read())
+        data16_size = headers[-1].size
+        self.assertEqual(len(base.raw_data), data16_size)
+        self.assertEqual(base.frame_rate, 192000)
+        self.assertEqual(base.sample_width, bit_depth / 8)
+
+        for test_file, bit_depth in test_files[1:]:
+            path = os.path.join(data_dir, test_file)
+            seg = AudioSegment.from_file(path)
+            self.assertEqual(seg.sample_width, bit_depth / 8)
+            self.assertEqual(seg.frame_rate, 192000)
+            self.assertEqual(len(seg.raw_data), len(base.raw_data) *
+                             seg.sample_width / base.sample_width)
+            self.assertEqual(seg.frame_rate, 192000)
+
     def test_concat(self):
         catted_audio = self.seg1 + self.seg2
 
@@ -284,6 +421,17 @@ class AudioSegmentTests(unittest.TestCase):
         past_end = second_long_slice[:1500]
         self.assertTrue(second_long_slice._data == past_end._data)
 
+    def test_slicing_by_step(self):
+        audio = self.seg1[:10000]
+        chunks = audio[:0]
+
+        for chunk in audio[::1000]:
+            self.assertTrue(isinstance(chunk, AudioSegment))
+            self.assertEqual(len(chunk), 1000)
+            chunks += chunk
+
+        self.assertEqual(len(audio), len(chunks))
+
     def test_indexing(self):
         short = self.seg1[:100]
 
@@ -332,7 +480,6 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertEqual(seg_lchannel.frame_count(), seg.frame_count())
         self.assertEqual(seg_rchannel.frame_count(), seg.frame_count())
 
-
     def test_apply_gain_stereo(self):
         seg = self.seg1
 
@@ -414,7 +561,8 @@ class AudioSegmentTests(unittest.TestCase):
     def test_export_as_raw(self):
         seg = self.seg1
         exported_raw = seg.export(format='raw')
-        seg_exported_raw = AudioSegment.from_raw(exported_raw, sample_width=seg.sample_width, frame_rate=seg.frame_rate, channels = seg.channels)
+        seg_exported_raw = AudioSegment.from_raw(exported_raw, sample_width=seg.sample_width, frame_rate=seg.frame_rate,
+                                                 channels=seg.channels)
 
         self.assertWithinTolerance(len(seg_exported_raw),
                                    len(seg),
@@ -509,10 +657,12 @@ class AudioSegmentTests(unittest.TestCase):
             if sys.platform == 'win32':
                 tmp_mp3_file.close()
 
-            seg.export(tmp_mp3_file.name)
+            fd = seg.export(tmp_mp3_file.name)
+            fd.close()
 
             for i in range(3):
-                AudioSegment.from_mp3(tmp_mp3_file.name).export(tmp_mp3_file.name, "mp3")
+                fd = AudioSegment.from_mp3(tmp_mp3_file.name).export(tmp_mp3_file.name, "mp3")
+                fd.close()
 
             tmp_seg = AudioSegment.from_mp3(tmp_mp3_file.name)
             self.assertFalse(len(tmp_seg) < len(seg))
@@ -764,14 +914,12 @@ class AudioSegmentTests(unittest.TestCase):
         else:
             raise Exception("AudioSegment.invert_phase() didn't catch a bad input (mono)")
 
-
         s_inv = s.invert_phase()
         self.assertFalse(s == s_inv)
         self.assertTrue(s.rms == s_inv.rms)
         self.assertTrue(s == s_inv.invert_phase())
 
-
-        s_inv_right = s.invert_phase(channels=(0,1))
+        s_inv_right = s.invert_phase(channels=(0, 1))
         left, right = s_inv_right.split_to_mono()
 
         self.assertFalse(s_mono == s_inv_right)
@@ -779,7 +927,7 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertTrue(left == s_mono)
         self.assertFalse(right == s_mono)
 
-        s_inv_left = s.invert_phase(channels=(1,0))
+        s_inv_left = s.invert_phase(channels=(1, 0))
         left, right = s_inv_left.split_to_mono()
 
         self.assertFalse(s_mono == s_inv_left)
@@ -787,7 +935,6 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertFalse(left == s_mono)
         self.assertTrue(right == s_mono)
 
-
     def test_max_dBFS(self):
         sine_0_dbfs = Sine(1000).to_audio_segment()
         sine_minus_3_dbfs = Sine(1000).to_audio_segment(volume=-3.0)
@@ -836,6 +983,25 @@ class AudioSegmentTests(unittest.TestCase):
         seg1 = seg.remove_dc_offset(channel=1, offset=(-0.06))
         self.assertWithinTolerance(seg1.get_dc_offset(1), -0.1, tolerance=0.01)
 
+    def test_from_file_clean_fail(self):
+        delete = sys.platform != 'win32'
+        orig_tmpdir = gettempdir()
+        new_tmpdir = mkdtemp()
+        tempfile.tempdir = new_tmpdir
+
+        with NamedTemporaryFile('w+b', suffix='.wav', delete=delete) as tmp_wav_file:
+            tmp_wav_file.write("not really a wav".encode('utf-8'))
+            tmp_wav_file.flush()
+            self.assertRaises(CouldntDecodeError, AudioSegment.from_file, tmp_wav_file.name)
+            files = os.listdir(tempfile.tempdir)
+            self.assertEquals(files, [os.path.basename(tmp_wav_file.name)])
+
+        if sys.platform == 'win32':
+            os.remove(tmp_wav_file.name)
+
+        tempfile.tempdir = orig_tmpdir
+        os.rmdir(new_tmpdir)
+
 
 class SilenceTests(unittest.TestCase):
 
@@ -868,6 +1034,11 @@ class SilenceTests(unittest.TestCase):
         silent_ranges = detect_silence(self.seg1, min_silence_len=500, silence_thresh=-20)
         self.assertEqual(silent_ranges, [[0, 775], [3141, 4033], [5516, 6051]])
 
+    def test_detect_silence_seg1_with_seek_split(self):
+        silent_ranges = detect_silence(self.seg1, min_silence_len=500, silence_thresh=-20,
+                                       seek_step=10)
+        self.assertEqual(silent_ranges, [[0, 770], [3150, 4030], [5520, 6050]])
+
     def test_realistic_audio(self):
         silent_ranges = detect_silence(self.seg4, min_silence_len=1000, silence_thresh=self.seg4.dBFS)
 
@@ -974,7 +1145,7 @@ class NoConverterTests(unittest.TestCase):
         self.assertRaises(OSError, func)
 
     def test_init_AudioSegment_data_buffer(self):
-        seg = AudioSegment(data = "\0" * 34, sample_width=2, frame_rate=4, channels=1)
+        seg = AudioSegment(data="\0" * 34, sample_width=2, frame_rate=4, channels=1)
 
         self.assertEqual(seg.duration_seconds, 4.25)
 
@@ -982,25 +1153,20 @@ class NoConverterTests(unittest.TestCase):
 
         self.assertEqual(seg.frame_rate, 4)
 
-
     def test_init_AudioSegment_data_buffer_with_missing_args_fails(self):
-
-        func = partial(AudioSegment, data = "\0" * 16, sample_width=2, frame_rate=2)
+        func = partial(AudioSegment, data="\0" * 16, sample_width=2, frame_rate=2)
         self.assertRaises(MissingAudioParameter, func)
 
-        func = partial(AudioSegment, data = "\0" * 16, sample_width=2, channels=1)
+        func = partial(AudioSegment, data="\0" * 16, sample_width=2, channels=1)
         self.assertRaises(MissingAudioParameter, func)
 
-        func = partial(AudioSegment, data = "\0" * 16, frame_rate=2, channels=1)
+        func = partial(AudioSegment, data="\0" * 16, frame_rate=2, channels=1)
         self.assertRaises(MissingAudioParameter, func)
 
-
     def test_init_AudioSegment_data_buffer_with_bad_values_fails(self):
-
-        func = partial(AudioSegment, data = "\0" * 14, sample_width=4, frame_rate=2, channels=1)
+        func = partial(AudioSegment, data="\0" * 14, sample_width=4, frame_rate=2, channels=1)
         self.assertRaises(ValueError, func)
 
-
     def test_exporting(self):
         seg = AudioSegment.from_wav(self.wave_file)
         exported = AudioSegment.from_wav(seg.export(format="wav"))
@@ -1045,8 +1211,6 @@ class FilterTests(unittest.TestCase):
         self.assertAlmostEqual(less_treble.dBFS, s.dBFS, places=0)
 
 
-
-
 if __name__ == "__main__":
     import sys
 
